﻿using System;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Connect to ALM Server
    /// </summary>
    public class ALMConnection : IDisposable
    {
        private readonly string _urlAuthenticate;
        private readonly string _urlLogout;

        /// <summary>
        /// Server name of ALM
        /// </summary>
        public string ServerName { get; set; }

        /// <summary>
        /// Server port of ALM
        /// </summary>
        public string ServerPort { get; set; }

        /// <summary>
        /// Domain of ALM
        /// </summary>
        public string Domain { get; set; }

        /// <summary>
        /// Project of ALM
        /// </summary>
        public string Project { get; set; }

        /// <summary>
        /// Username of login ALM
        /// </summary>
        public string UserName { get; set; }

        /// <summary>
        /// Password of login ALM
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// HTTP object
        /// </summary>
        public HttpHelper Http { get; set; }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="serverName">Server Name</param>
        /// <param name="serverPort">Server Port</param>
        /// <param name="domain">Domain</param>
        /// <param name="project">Project</param>
        /// <param name="username">User name</param>
        /// <param name="password">Password</param>
        public ALMConnection(string serverName, string serverPort, string domain, string project, string username, string password)
        {
            ServerName = serverName;
            ServerPort = serverPort;
            Domain = domain;
            Project = project;
            UserName = username;
            Password = password;

            _urlAuthenticate = string.Format("http://{0}:{1}/qcbin/authentication-point/authenticate", serverName, serverPort);
            _urlLogout = string.Format("http://{0}:{1}/qcbin/authentication-point/logout", serverName, serverPort);

            try
            {
                Http = new HttpHelper(username, password)
                           {
                               Cache = System.Net.Cache.RequestCacheLevel.NoCacheNoStore,
                               ContentType = "application/xml",
                               Method = "GET"
                           };
                Http.Request(_urlAuthenticate);
                var token = Common.ParseHeaders(Http.ResponseHeaders["Set-Cookie"], "LWSSO_COOKIE_KEY");
                Http.RequestHeaders.Add("Cookie", string.Format("LWSSO_COOKIE_KEY={0}", token));
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        /// <summary>
        /// Log off from ALM
        /// </summary>
        public void Logoff()
        {
            try
            {
                Http.ContentType = "application/xml";
                Http.Method = "GET";
                Http.Request(_urlLogout);
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        void IDisposable.Dispose()
        {
            Logoff();
        }
    }
}
