﻿using System;
using System.Collections.Generic;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// defect link collection class
    /// </summary>
    public class ALMDefectLinks : ALMEntityCollection<ALMDefectLink>
    {
        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="connection">connection</param>
        public ALMDefectLinks(ALMConnection connection)
            : base(connection)
        {
        }

        #region public methods
        /// <summary>
        /// Get defect link
        /// </summary>
        /// <param name="firstId">First endpoint  Id</param>
        /// <param name="secondId">Second endpoint Id</param>
        /// <param name="secondType">Second endpoint type</param>
        /// <returns>Defect Link</returns>
        public ALMDefectLink Get(int firstId, int secondId, string secondType)
        {
            var filters = new Dictionary<string, string>
                              {
                                  {"first-endpoint-id", firstId.ToString()},
                                  {"second-endpoint-id", secondId.ToString()},
                                  {"second-endpoint-type", secondType}
                              };
            var dls = base.GetCollection(filters);

            if (dls.Count == 0) throw new Exception("Cannot get the defect link.");
            if (dls.Count > 1) throw new Exception("Get Multiple defect links.");

            var bugLink = dls[0];

            return bugLink;
        }

        /// <summary>
        /// Create defect link
        /// </summary>
        /// <param name="firstId">First endpoint  Id</param>
        /// <param name="secondId">Second endpoint Id</param>
        /// <param name="secondType">Second endpoint type</param>
        /// <returns>Defect Link</returns>
        public ALMDefectLink Create(int firstId, int secondId, string secondType)
        {
            var entityFields = new Dictionary<string, string>
                                   {
                                       {"first-endpoint-id", firstId.ToString()},
                                       {"second-endpoint-id", secondId.ToString()},
                                       {"second-endpoint-type", secondType}
                                   };

            var buglink = base.Create(entityFields);

            return buglink;
        }        
        #endregion

        #region private methods
        #endregion
    }

    /// <summary>
    /// Defect link class
    /// </summary>
    public class ALMDefectLink: ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods   
        #endregion
    }
}
