﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Entity Field class
    /// </summary>
    public class ALMCustomizedField
    {
        /// <summary>
        /// Field name
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Field Label name
        /// </summary>
        public string Label { get; set; }

        /// <summary>
        /// Is required field
        /// </summary>
        public bool IsRequired { get; set; }

        /// <summary>
        /// Field value
        /// </summary>
        public string Value { get; set; }

        /// <summary>
        /// Is to update field
        /// </summary>
        public bool IsToUpdate { get; set; }

        /// <summary>
        /// Get field for specified entity type.
        /// </summary>
        /// <param name="conn">connection</param>
        /// <param name="entityType">entity type</param>
        /// <returns>list of fields</returns>
        public static List<ALMCustomizedField> GetFields(ALMConnection conn, string entityType)
        {
            var fields = new List<ALMCustomizedField>();

            var url = string.Format("http://{0}:{1}/qcbin/rest/domains/{2}/projects/{3}/customization/entities/{4}/fields", conn.ServerName, conn.ServerPort, conn.Domain, conn.Project, entityType);

            try
            {
                conn.Http.Method = "GET";
                conn.Http.ContentType = "application/xml";
                var status = conn.Http.Request(url);
                if (status >= 200 && status <= 204)
                {
                    if (!string.IsNullOrEmpty(conn.Http.Response))
                    {
                        var doc = new XmlDocument();
                        doc.LoadXml(conn.Http.Response);
                        var list = doc.SelectNodes(@"Fields/Field");
                        fields.AddRange(from XmlNode node in list
                                        select new ALMCustomizedField
                                                   {
                                                       Name = node.Attributes["Name"].Value, Label = node.Attributes["Label"] == null ? string.Empty : node.Attributes["Label"].Value, IsRequired = Convert.ToBoolean(node.SelectSingleNode(@"Required").InnerText), IsToUpdate = false
                                                   });
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

            return fields;
        }
    }
}
