﻿using System;
using System.Collections.Generic;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test run collection class
    /// </summary>
    public class ALMRuns : ALMEntityCollection<ALMRun>
    {
        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="connection">connection</param>
        public ALMRuns(ALMConnection connection)
            : base(connection)
        { }

        #region Public methods
        /// <summary>
        /// Create Test Run
        /// </summary>
        /// <param name="instance">test instance</param>
        /// <returns>Test Instance</returns>
        public ALMRun Create(ALMTestInstance instance)
        {
            var now = DateTime.UtcNow;

            var fields = new Dictionary<string, string>
                             {
                                 {"test-id", instance.GetField("test-id").Value},
                                 {"cycle-id", instance.GetField("cycle-id").Value},
                                 {"testcycl-id", instance.GetField("id").Value},
                                 {"status", "No Run"},
                                 {
                                     "name",
                                     string.Format("Run_{0}-{1}_{2}-{3}-{4}", now.Month, now.Day, now.Hour, now.Minute,
                                                   now.Second)
                                     },
                                 {"owner", Connection.UserName}
                             };
            var instType =instance.GetField("subtype-id").Value;
            fields.Add("subtype-id", string.Format("hp.qc.run.{0}", instType.Substring(instType.LastIndexOf(".") + 1, instType.Length - instType.LastIndexOf(".") - 1)));

            return base.Create(fields);
        }
        #endregion
    }

    /// <summary>
    /// Run class
    /// </summary>
    public class ALMRun : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods
        /// <summary>
        /// Update Test Rim
        /// </summary>
        /// <returns></returns>
        public ALMRun Update()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            return new ALMRuns(Connection).Update(Id.Value, GetFieldsDictionary());
        }

        public ALMRun Pass()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "Passed");
            return Update();
        }

        public ALMRun Fail()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "Failed");
            return Update();
        }

        public ALMRun NotComplete()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "Not Completed");
            return Update();
        }

        public ALMRun Block()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "Blocked");
            return Update();
        }

        public ALMRun Warn()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "Warning");
            return Update();
        }

        public ALMRun ReSet()
        {
            if (Id <= 0) throw new Exception("Test Run Id cannot be empty.");

            SetUpdateField("status", "No Run");
            return Update();
        }
        #endregion


    }
}
