﻿using System;
using System.Collections.Generic;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test Collection class
    /// </summary>
    public class ALMTests : ALMEntityCollection<ALMTest>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="connection">connection</param>
        public ALMTests(ALMConnection connection)
            : base(connection)
        {
        }

        #region public methods
        /// <summary>
        /// Create Test
        /// </summary>
        /// <param name="testFolderId">test folder Id</param>
        /// <param name="name">Test Name</param>
        /// <returns>Test</returns>
        public ALMTest Create(int testFolderId, string name)
        {
            if (string.IsNullOrEmpty(name)) throw new Exception("Test name cannot be empty.");

            var fields = new Dictionary<string, string>
                             {
                                 {"name", name},
                                 {"parent-id", testFolderId.ToString()},
                                 {"subtype-id", "MANUAL"},
                                 {"status", "Ready"},
                             };

            return base.Create(fields);
        }
        #endregion
    }

    /// <summary>
    /// Test class
    /// </summary>
    public class ALMTest : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods
        /// <summary>
        /// Update Test
        /// </summary>
        /// <returns></returns>
        public ALMTest Update()
        {
            if (Id <= 0) throw new Exception("Test Id cannot be empty.");

            return new ALMTests(Connection).Update(Id.Value, GetFieldsDictionary());
        }

        /// <summary>
        /// Get steps for the test
        /// </summary>
        /// <returns></returns>
        public List<ALMTestStep> GetSteps()
        {
            if (Id <= 0) throw new Exception("Test Id cannot be empty.");

            var queries = new Dictionary<string, string> { { "parent-id", Id.ToString() }};

            var teststeps = new ALMTestSteps(Connection);
            return teststeps.GetCollection(queries);
        }

        /// <summary>
        /// Get steps for the test
        /// </summary>
        /// <returns></returns>
        public ALMTestStep GetStep(int order)
        {
            if (Id <= 0) throw new Exception("Test Id cannot be empty.");

            var queries = new Dictionary<string, string> { { "parent-id", Id.ToString() }, { "step-order", order.ToString() } };

            var teststeps = new ALMTestSteps(Connection);
            var teststepcollection = teststeps.GetCollection(queries);

            if (teststepcollection == null)
                throw new Exception(string.Format("Cannot find the step '{0}' in this test", order.ToString()));
            if (teststepcollection.Count == 0)
                throw new Exception(string.Format("Cannot find the step '{0}' in this test", order.ToString()));

            return teststepcollection[0];
        }

        /// <summary>
        /// add a new step for the test
        /// </summary>
        /// <param name="stepName"></param>
        /// <param name="description"></param>
        /// <param name="expected"></param>
        /// <returns></returns>
        public ALMTestStep AddStep(string stepName, string description, string expected)
        {
            if (Id <= 0) throw new Exception("Test Id cannot be empty.");

            var teststeps = new ALMTestSteps(Connection);
            var newteststep = teststeps.Create(Id.Value, stepName, description, expected);

            return newteststep;
        }
        #endregion

    }
}
