﻿using System;
using System.Collections.Generic;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test folder collection class
    /// </summary>
    public class ALMTestFolders : ALMEntityCollection<ALMTestFolder>
    {
         /// <summary>
        /// constructor
        /// </summary>
        public ALMTestFolders(ALMConnection connection)
            : base(connection)
        {
            
        }

        #region public methods
        /// <summary>
        /// Create test folder
        /// </summary>
        /// <param name="parentId">parent id</param>
        /// <param name="folderName">folder name</param>
        /// <returns>test folder</returns>
        public ALMTestFolder Create(int parentId, string folderName)
        {
            if (string.IsNullOrEmpty(folderName)) throw new Exception("Folder name of test cannot be empty.");

            var fields = new Dictionary<string, string> {{"name", folderName}, {"parent-id", parentId.ToString()}};

            return base.Create(fields);
        }
        #endregion

        #region private methods
        #endregion

    }

    /// <summary>
    /// Test folder class
    /// </summary>
    public class ALMTestFolder : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods        
        /// <summary>
        /// Find test under the test folder
        /// </summary>
        /// <param name="testName">test name</param>
        /// <returns>test</returns>
        public ALMTest FindTest(string testName)
        {
            var queries = new Dictionary<string, string> {{"parent-id", Id.ToString()}, {"name", testName}};

            var tests = new ALMTests(Connection);
            var testList = tests.GetCollection(queries);
            return testList.Count == 0 ? null : testList[0];
        }

        /// <summary>
        /// Create test folder under the test folder
        /// </summary>
        /// <param name="folderName">folder name</param>
        /// <returns>Test folder</returns>
        public ALMTestFolder CreateFolder(string folderName)
        {
            var testfolders = new ALMTestFolders(Connection);
            return testfolders.Create(Id.Value, folderName);
        }

        /// <summary>
        /// Add Test under the test folder
        /// </summary>
        /// <param name="testName">test name</param>
        /// <returns>Test</returns>
        public ALMTest AddTest(string testName)
        {
            var tests = new ALMTests(Connection);
            return tests.Create(Id.Value, testName);
        }

        /// <summary>
        /// Update Test folder
        /// </summary>
        /// <returns></returns>
        public ALMTestFolder Update()
        {
            if (Id <= 0) throw new Exception("Test Folder Id cannot be empty.");

            return new ALMTestFolders(Connection).Update(Id.Value, GetFieldsDictionary());
        }
        #endregion

        #region private methods
        #endregion
    }
}
