﻿using System.Collections.Generic;
using System.Linq;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test instance collection class
    /// </summary>
    public class ALMTestInstances : ALMEntityCollection<ALMTestInstance>
    {
        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="connection">connection</param>
        public ALMTestInstances(ALMConnection connection)
            : base(connection)
        {            
        }

        #region Public methods
        /// <summary>
        /// Create Test Instance
        /// </summary>
        /// <param name="testId">test Id</param>
        /// <param name="testSetId">Test Set Id</param>
        /// <param name="order">test Set order number</param>
        /// <param name="type">Test type, normally is MANUAL</param>
        /// <returns>Test Instance</returns>
        public ALMTestInstance Create(int testId, int testSetId, int order, string type)
        {
            var fields = new Dictionary<string, string>
                             {
                                 {"test-id", testId.ToString()},
                                 {"cycle-id", testSetId.ToString()},
                                 {"test-order", order.ToString()},
                                 {"subtype-id", string.Format("hp.qc.test-instance.{0}", type)}
                             };

            return base.Create(fields);
        }
        #endregion
    }

    /// <summary>
    /// test instance class
    /// </summary>
    public class ALMTestInstance : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods 
        /// <summary>
        /// Get Last test run
        /// </summary>
        /// <returns>Test run</returns>
        public ALMRun GetLastRun()
        {
            var queries = new Dictionary<string, string>
                              {
                                  {"cycle-id", GetField("cycle-id").Value},
                                  {"test-instance", GetField("test-instance").Value},
                                  {"test-id", GetField("test-id").Value}
                              };

            var runs = new ALMRuns(Connection);
            var runlist = runs.GetCollection(queries);
            runlist.Sort(delegate(ALMRun r1, ALMRun r2)
            {
                var r1Id = (r1.Id == null ? 0 : r1.Id.Value);
                var r2Id = (r2.Id == null ? 0 : r2.Id.Value);
                return r2Id.CompareTo(r1Id);
            });
            var lastrun = runlist.First();

            return lastrun;
        }
        #endregion
    }
}
