﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test set collection class
    /// </summary>
    public class ALMTestSets : ALMEntityCollection<ALMTestSet>
    {
        /// <summary>
        /// constructor
        /// </summary>
        public ALMTestSets(ALMConnection connection) : base(connection)
        {
        }

        #region public methods
        /// <summary>
        /// Create Test Set
        /// </summary>
        /// <param name="testSetFolderId">test set folder Id</param>
        /// <param name="name">Test Name</param>
        /// <returns>Test</returns>
        public ALMTestSet Create(int testSetFolderId, string name)
        {
            if (string.IsNullOrEmpty(name)) throw new Exception("Test name cannot be empty.");

            var fields = new Dictionary<string, string> { { "name", name }, { "parent-id", testSetFolderId.ToString() }, { "subtype-id", "hp.qc.test-set.default" }, {"status", "Open" } };

            return base.Create(fields);
        }
        #endregion
    }

    /// <summary>
    /// Test set class
    /// </summary>
    public class ALMTestSet : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods
        /// <summary>
        /// Update Test SET
        /// </summary>
        /// <returns></returns>
        public ALMTestSet Update()
        {
            if (Id <= 0) throw new Exception("Test Set Id cannot be empty.");

            return new ALMTestSets(Connection).Update(Id.Value, GetFieldsDictionary());
        }

        /// <summary>
        /// Add tests under the test set
        /// </summary>
        /// <param name="tests">test list</param>
        public void AddTests(List<ALMTest> tests)
        {
            if (tests.Count == 0) throw new Exception("Tests cannot be empty.");

            var testinstances = new ALMTestInstances(Connection);
            var i = 1;
            foreach (var test in tests)
            {
                testinstances.Create(test.Id.Value, Id.Value, i, test.GetField("subtype-id").Value);
                i++;
            }
        }

        /// <summary>
        /// Get test instance list
        /// </summary>
        /// <returns>test instance list</returns>
        public List<ALMTestInstance> GetTestInstances()
        {
            var testinstances = new ALMTestInstances(Connection);
            var filter = new Dictionary<string, string> {{"cycle-id", Id.Value.ToString()}};
            return testinstances.GetCollection(filter);
        }

        /// <summary>
        /// Get test instance
        /// </summary>
        /// <param name="testId">test id</param>
        /// <param name="testInstance">test instance</param>
        /// <returns>test instance</returns>
        public ALMTestInstance GetTestInstance(int testId, int testInstance)
        {
            var testinstances = new ALMTestInstances(Connection);
            var filter = new Dictionary<string, string> { { "cycle-id", Id.Value.ToString() }, { "test-id", testId.ToString() }, { "test-instance", testInstance.ToString() } };
            var testInstanceList = testinstances.GetCollection(filter);
            if (testInstanceList.Count == 1)
                return testInstanceList[0];
            throw new Exception(string.Format("Found multiple/none test instance with TestSet ID={0}, Test ID={1}, Test Instance={2}", Id.Value, testId, testInstance));
        }

        /// <summary>
        /// Run all tests under the test set
        /// </summary>
        /// <returns>run list</returns>
        public List<ALMRun> RunTests()
        {
            var testInstances = GetTestInstances();
            var runs = new ALMRuns(Connection);

            return testInstances.Select(ati => runs.Create(ati)).ToList();
        }

        /// <summary>
        /// Get last runs for the test set
        /// </summary>
        /// <returns>run list</returns>
        public List<ALMRun> GetTestLastRuns()
        {
            var testInstances = GetTestInstances();

            return testInstances.Select(testins => testins.GetLastRun()).ToList();
        }
        #endregion
    }
}
