﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test set Folder collection class
    /// </summary>
    public class ALMTestSetFolders : ALMEntityCollection<ALMTestSetFolder>
    {
        /// <summary>
        /// constructor
        /// </summary>
        public ALMTestSetFolders(ALMConnection connection)
            : base(connection)
        {
        }

        #region public methods
        /// <summary>
        /// Create test set folder
        /// </summary>
        /// <param name="parentId">parent id</param>
        /// <param name="folderName">folder name</param>
        /// <returns>test folder</returns>
        public ALMTestSetFolder Create(int parentId, string folderName)
        {
            if (string.IsNullOrEmpty(folderName)) throw new Exception("Folder name of testset cannot be empty.");

            var fields = new Dictionary<string, string> {{"name", folderName}, {"parent-id", parentId.ToString()}};

            return base.Create(fields);
        }
        #endregion

        #region private methods
        #endregion
    }

    /// <summary>
    /// Test set folder class
    /// </summary>
    public class ALMTestSetFolder : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods 
        /// <summary>
        /// Find test under the test set folder
        /// </summary>
        /// <param name="testSetName">test set name</param>
        /// <returns>test set</returns>
        public ALMTestSet FindTestSet(string testSetName)
        {
            var queries = new Dictionary<string, string> {{"parent-id", Id.ToString()}};

            var testsets = new ALMTestSets(Connection);
            var testSetList = testsets.GetCollection(queries);
            if (testSetList.Count == 0) throw new Exception(string.Format("No test set be found under test set folder: {0}.", GetField("name").Value));

            var foundcount = testSetList.Where(o => o.GetField("name").Value.Equals(testSetName)).Count();

            if (foundcount == 0) throw new Exception(string.Format("Cannot find the test set: {0} ", testSetName));
            if (foundcount > 1) throw new Exception(string.Format("Found multiple test set: {0} ", testSetName));

            var testSet = testSetList.Where(o => o.GetField("name").Value.Equals(testSetName)).First();

            return testSet;
        }

        /// <summary>
        /// Create test folder under the test folder
        /// </summary>
        /// <param name="folderName">folder name</param>
        /// <returns>Test folder</returns>
        public ALMTestSetFolder CreateFolder(string folderName)
        {
            var testsetfolders = new ALMTestSetFolders(Connection);
            return testsetfolders.Create(Id.Value, folderName);
        }

        /// <summary>
        /// Add Test under the test folder
        /// </summary>
        /// <param name="testSetName">new test set name</param>
        /// <returns>Test Set</returns>
        public ALMTestSet AddTestSet(string testSetName)
        {
            var testSets = new ALMTestSets(Connection);
            return testSets.Create(Id.Value, testSetName);
        }

        /// <summary>
        /// Update Test set folder
        /// </summary>
        /// <returns></returns>
        public ALMTestSetFolder Update()
        {
            if (Id <= 0) throw new Exception("Test Set Folder Id cannot be empty.");

            return new ALMTestSetFolders(Connection).Update(Id.Value, GetFieldsDictionary());
        }
        #endregion

        #region private methods
        #endregion
    }
}
