﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HPALMRestAPIWrapper;

namespace HPALMRestAPIWrapper
{
    /// <summary>
    /// Test Steps Collection class
    /// </summary>
    public class ALMTestSteps : ALMEntityCollection<ALMTestStep>
    {
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="connection">connection</param>
        public ALMTestSteps(ALMConnection connection)
            : base(connection)
        {
        }

        #region public methods
        /// <summary>
        /// Create Test Step
        /// </summary>
        /// <param name="testId">test id</param>
        /// <param name="stepName">step name</param>
        /// <param name="description">description of step</param>
        /// <param name="expected">expected result of step</param>
        /// <returns></returns>
        public ALMTestStep Create(int testId, string stepName, string description, string expected)
        {
            if (testId == 0) throw new Exception("Test id cannot be empty.");

            var fields = new Dictionary<string, string>
                             {
                                 {"parent-id", testId.ToString()},
                                 {"name", stepName},
                                 {"description", description},
                                 {"expected", expected},
                             };

            return base.Create(fields);
        }
        #endregion
    }

    /// <summary>
    /// Test step class
    /// </summary>
    public class ALMTestStep : ALMEntity
    {
        #region private members
        #endregion

        #region public Properties
        #endregion

        #region constructor
        #endregion

        #region public methods

        /// <summary>
        /// Update Test
        /// </summary>
        /// <returns></returns>
        public ALMTestStep Update()
        {
            if (Id <= 0) throw new Exception("Test Step Id cannot be empty.");

            return new ALMTestSteps(Connection).Update(Id.Value, GetFieldsDictionary());
        }

        /// <summary>
        /// Update Test
        /// </summary>
        /// <returns></returns>
        public ALMTestStep Update(string stepName, string description, string expected)
        {
            if (Id <= 0) throw new Exception("Test Step Id cannot be empty.");

            if (!string.IsNullOrEmpty(stepName)) SetUpdateField("name", stepName);
            if (!string.IsNullOrEmpty(description)) SetUpdateField("description", description);
            if (!string.IsNullOrEmpty(expected)) SetUpdateField("expected", expected);

            return this.Update();
        }


        public ALMTestStep MoveUp()
        {
            if (Id <= 0) throw new Exception("Test Step Id cannot be empty.");

            int currentStepOrder = Convert.ToInt32(GetField("step-order").Value);
            int newStepOrder = currentStepOrder-1;
            if ( currentStepOrder == 1)
                throw new Exception("The step is first step, cannot move up.");

            int testId = Convert.ToInt32(GetField("parent-id").Value);
            var test = new ALMTests(Connection).Get(testId);

            var switchStep = test.GetStep(newStepOrder);
            switchStep.SetUpdateField("step-order", currentStepOrder.ToString());
            switchStep.Update();

            SetUpdateField("step-order", newStepOrder.ToString());
            return this.Update();
        }

        public ALMTestStep MoveDown()
        {
            if (Id <= 0) throw new Exception("Test Step Id cannot be empty.");

            int currentStepOrder = Convert.ToInt32(GetField("step-order").Value);
            int newStepOrder = currentStepOrder + 1;

            int testId = Convert.ToInt32(GetField("parent-id").Value);
            var test = new ALMTests(Connection).Get(testId);
            int stepcount = test.GetSteps().Count;

            if (currentStepOrder == stepcount-1)
                throw new Exception("The step is last step, cannot move down.");

            var switchStep = test.GetStep(newStepOrder);
            switchStep.SetUpdateField("step-order", currentStepOrder.ToString());
            switchStep.Update();

            SetUpdateField("step-order", newStepOrder.ToString());
            return this.Update();
        }
        #endregion

        #region private methods
        #endregion

    }
}
