﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;

namespace HPALMRestAPIWrapper
{
    public class Common
    {
        public static string ParseHeaders(string str, string sessionname)
        {
            var startstr = sessionname;

            if (str.IndexOf(startstr) < 0)
                return null;

            var startpos = str.IndexOf(startstr) + startstr.Length + 1;
            return str.Substring(startpos, str.Length - startpos);
        }

        public static string BuildFilters(Dictionary<string, string> filters)
        {
            var query = "query={";
            string con = null;

            if (filters == null)
                return null;

            foreach (var f in filters)
            {
                if (string.IsNullOrEmpty(con))
                    con += string.Format("{0}[{1}]", f.Key, f.Value.IndexOf(" ") > 0 ? string.Format("'{0}'", f.Value) : f.Value);
                else
                    con += string.Format(";{0}[{1}]", f.Key, f.Value.IndexOf(" ") > 0 ? string.Format("'{0}'", f.Value) : f.Value);
            }

            query += con + "}";

            return query;
        }

        public static string BuildFields(string[] fields)
        {
            string fieldsquery = null;

            if (fields == null)
                return null;

            if (fields.Count() > 0)
            {
                fieldsquery = fields.Aggregate("fields=", (current, f) => current + string.Format("{0},", f));
            }
            
            return fieldsquery;
        }

        public static string GetEntityType(string classname)
        {
            string name = null;
            switch (classname)
            {
                case "ALMTest":
                    name = "tests";
                    break;
                case "ALMTestSet":
                    name = "test-sets";
                    break;
                case "ALMDefect":
                    name = "defects";
                    break;
                case "ALMRequirement":
                    name = "requirements";
                    break;
                case "ALMRun":
                    name = "runs";
                    break;
                case "ALMTestFolder":
                    name = "test-folders";
                    break;
                case "ALMTestSetFolder":
                    name = "test-set-folders";
                    break;
                case "ALMTestInstance":
                    name = "test-instances";
                    break;
                case "ALMDefectLink":
                    name = "defect-links";
                    break;
                case "ALMTestStep":
                    name = "design-steps";
                    break;
            }

            return name;
        }

        public static string GetFieldValue(XmlDocument xmlDoc, string fieldname)
        {
            string fieldValue = null;

            if (xmlDoc == null)
                throw new Exception("XMLDoc is null");

            try
            {
                var node = xmlDoc.SelectSingleNode(string.Format("Fields/Field[@Name='{0}']", fieldname));
                if (node != null)
                {
                    var valueNode = node.SelectSingleNode("Value");
                    if (valueNode != null)
                        fieldValue = valueNode.InnerText;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

            return fieldValue;
        }

        public static string GetQCRestException(string response)
        {
            string exceptionTitle;

            try
            {
                var doc = new XmlDocument();
                doc.LoadXml(response);

                exceptionTitle = doc.SelectSingleNode(@"QCRestException/Title").InnerText;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
            
            return exceptionTitle;
        }

        public static string BuildEntityXML(Dictionary<string, string> fields, string entityType)
        {
            var doc = CreateXmlDocument("Entity");
            var rootNode = doc.SelectSingleNode("Entity");
            CreateAttribute(rootNode, "Type", entityType);

            var fieldsNode = CreateChildNode(rootNode, "Fields");

            foreach (var kvp in fields)
            {
                var fieldNode = CreateChildNode(fieldsNode, "Field", "Name", kvp.Key);
                if (!string.IsNullOrEmpty(kvp.Value))
                    CreateChildNodeValue(fieldNode, "Value", kvp.Value);
            }

            return doc.InnerXml;            
        }

        public static void SetDictionaryValue(ref Dictionary<string, string> dic, string key, string value)
        {
            if (dic.ContainsKey(key))
                dic[key] = value;
            else
                dic.Add(key, value);
        }

        #region private methods        
        private static XmlAttribute CreateAttribute(XmlNode node, string attributeName, string value)
        {
            var doc = node.OwnerDocument;
            // create new attribute
            var attr = doc.CreateAttribute(attributeName);
            attr.Value = value;
            // link attribute to node
            node.Attributes.SetNamedItem(attr);
            return attr;
        }

        private static XmlDocument CreateXmlDocument(string rootName)
        {
            var doc = new XmlDocument();
            //var decl = doc.CreateXmlDeclaration("1.0", "utf-8", "");
            //doc.InsertBefore(decl, doc.DocumentElement);
            XmlNode newNode = doc.CreateElement(rootName);
            doc.AppendChild(newNode);
            return doc;
        }

        private static XmlNode CreateChildNode(XmlNode rootNode, string name)
        {
            var doc = rootNode.OwnerDocument;
            XmlNode newNode = doc.CreateElement(name);
            rootNode.AppendChild(newNode);

            return newNode;
        }

        private static XmlNode CreateChildNode(XmlNode rootNode, string nodeName, string attributeName, string attributeValue)
        {
            var doc = rootNode.OwnerDocument;
            XmlNode newNode = doc.CreateElement(nodeName);
            CreateAttribute(newNode, attributeName, attributeValue);
            rootNode.AppendChild(newNode);

            return newNode;
        }

        private static XmlNode CreateChildNodeValue(XmlNode rootNode, string name, string value)
        {
            var doc = rootNode.OwnerDocument;
            XmlNode newNode = doc.CreateElement(name);
            newNode.InnerText = value;
            rootNode.AppendChild(newNode);

            return newNode;
        }
        #endregion
    }
}
